<?php
/**
 * $Header: /repository/pear/Log/Log/display.php,v 1.9 2006/06/29 07:09:21 jon Exp $
 *
 * @version $Revision: 1.9 $
 * @package Log
 */

/**
 * The Log_display class is a concrete implementation of the Log::
 * abstract class which writes message into browser in usual PHP maner.
 * This may be useful because when you use PEAR::setErrorHandling in
 * PEAR_ERROR_CALLBACK mode error messages are not displayed by
 * PHP error handler.
 *
 * @author  Paul Yanchenko <pusher@inaco.ru>
 * @since   Log 1.8.0
 * @package Log
 *
 * @example display.php     Using the display handler.
 */
class Log_display extends Log
{
    /**
     * String to output before an error message
     * @var string
     * @access private
     */
    var $_error_prepend = '';

    /**
     * String to output after an error message
     * @var string
     * @access private
     */
    var $_error_append = '';

    /**
     * String used to represent a line break.
     * @var string
     * @access private
     */
    var $_linebreak = "<br />\n";

    /**
     * Constructs a new Log_display object.
     *
     * @param string $name     Ignored.
     * @param string $ident    The identity string.
     * @param array  $conf     The configuration array.
     * @param int    $level    Log messages up to and including this level.
     * @access public
     */
    function __construct($name = '', $ident = '', $conf = array(),
                         $level = PEAR_LOG_DEBUG)
    {
        if(!class_exists('G')){
          $realdocuroot = str_replace( '\\', '/', $_SERVER['DOCUMENT_ROOT'] );
          $docuroot = explode( '/', $realdocuroot );
          array_pop( $docuroot );
          $pathhome = implode( '/', $docuroot ) . '/';
          array_pop( $docuroot );
          $pathTrunk = implode( '/', $docuroot ) . '/';
          require_once($pathTrunk.'gulliver/system/class.g.php');
        }
        $this->_id = G::encryptOld(microtime());
        $this->_ident = $ident;
        $this->_mask = Log::UPTO($level);

        if (isset($conf['error_prepend'])) {
            $this->_error_prepend = $conf['error_prepend'];
        } else {
            $this->_error_prepend = ini_get('error_prepend_string');
        }

        if (isset($conf['error_append'])) {
            $this->_error_append = $conf['error_append'];
        } else {
            $this->_error_append = ini_get('error_append_string');
        }

        if (isset($conf['linebreak'])) {
            $this->_linebreak = $conf['linebreak'];
        }
    }

    /**
     * Opens the display handler.
     *
     * @access  public
     * @since   Log 1.9.6
     */
    function open()
    {
        $this->_opened = true;
        return true;
    }

    /**
     * Closes the display handler.
     *
     * @access  public
     * @since   Log 1.9.6
     */
    function close()
    {
        $this->_opened = false;
        return true;
    }

    /**
     * Writes $message to the text browser. Also, passes the message
     * along to any Log_observer instances that are observing this Log.
     *
     * @param mixed  $message    String or object containing the message to log.
     * @param string $priority The priority of the message.  Valid
     *                  values are: PEAR_LOG_EMERG, PEAR_LOG_ALERT,
     *                  PEAR_LOG_CRIT, PEAR_LOG_ERR, PEAR_LOG_WARNING,
     *                  PEAR_LOG_NOTICE, PEAR_LOG_INFO, and PEAR_LOG_DEBUG.
     * @return boolean  True on success or false on failure.
     * @access public
     */
    function log($message, $priority = null)
    {
        /* If a priority hasn't been specified, use the default value. */
        if ($priority === null) {
            $priority = $this->_priority;
        }

        /* Abort early if the priority is above the maximum logging level. */
        if (!$this->_isMasked($priority)) {
            return false;
        }

        /* Extract the string representation of the message. */
        $message = $this->_extractMessage($message);

        /* Build and output the complete log line. */
        $realdocuroot = str_replace( '\\', '/', $_SERVER['DOCUMENT_ROOT'] );
        $docuroot = explode( '/', $realdocuroot );
        array_pop( $docuroot );
        $pathhome = implode( '/', $docuroot ) . '/';
        array_pop( $docuroot );
        $pathTrunk = implode( '/', $docuroot ) . '/';
        require_once($pathTrunk.'gulliver/system/class.inputfilter.php');
        $filter = new InputFilter();
        $tag    = $filter->xssFilterHard(ucfirst($this->priorityToString($priority)));
        echo $this->_error_prepend .
             '<b>' . $tag . '</b>: '.
             nl2br(htmlspecialchars($message)) .
             $this->_error_append . $this->_linebreak;

        /* Notify observers about this log message. */
        $this->_announce(array('priority' => $priority, 'message' => $message));

        return true;
    }

}
